@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -396,7 +393,7 @@ decode_qstring (unsigned char **in, unsigned char *inend)
 	if (inptr == inend)
 		return NULL;
 	
-	if (strncmp (inptr, "NIL", 3) != 0) {
+	if (strncmp ((const char *) inptr, "NIL", 3) != 0) {
 		g_assert (*inptr == '"');
 		inptr++;
 		start = inptr;
@@ -406,7 +403,7 @@ decode_qstring (unsigned char **in, unsigned char *inend)
 			inptr++;
 		}
 		
-		qstring = g_strndup (start, inptr - start);
+		qstring = g_strndup ((const char *) start, inptr - start);
 		unescape_qstring (qstring);
 		g_assert (*inptr == '"');
 		inptr++;
@@ -442,7 +439,6 @@ decode_params (unsigned char **in, unsigned char *inend)
 {
 	GMimeParam *params, *tail, *n;
 	unsigned char *inptr;
-	char *buf;
 	
 	inptr = *in;
 	params = NULL;
