@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -42,6 +43,17 @@
 #define d(x)
 
 
+/**
+ * SECTION: internet-address
+ * @title: InternetAddress
+ * @short_description: Internet addresses
+ * @see_also:
+ *
+ * An #InternetAddress represents what is commonly referred to as an
+ * E-Mail address.
+ **/
+
+
 /**
  * internet_address_new:
  *
@@ -50,7 +62,7 @@
  * Returns a new #InternetAddress object.
  **/
 InternetAddress *
-internet_address_new ()
+internet_address_new (void)
 {
 	InternetAddress *ia;
 	
@@ -73,17 +85,15 @@ internet_address_new ()
 static void
 internet_address_destroy (InternetAddress *ia)
 {
-	if (ia) {
-		g_free (ia->name);
-		
-		if (ia->type == INTERNET_ADDRESS_GROUP) {
-			internet_address_list_destroy (ia->value.members);
-		} else {
-			g_free (ia->value.addr);
-		}
-		
-		g_free (ia);
+	g_free (ia->name);
+	
+	if (ia->type == INTERNET_ADDRESS_GROUP) {
+		internet_address_list_destroy (ia->value.members);
+	} else {
+		g_free (ia->value.addr);
 	}
+	
+	g_free (ia);
 }
 
 
@@ -252,7 +262,10 @@ internet_address_add_member (InternetAddress *ia, InternetAddress *member)
  * internet_address_get_type:
  * @ia: internet address
  *
- * Returns the type of the internet address.
+ * Gets the type of the internet address, which will either be
+ * #INTERNET_ADDRESS_NAME or #INTERNET_ADDRESS_GROUP.
+ *
+ * Returns the type of @ia.
  **/
 InternetAddressType
 internet_address_get_type (InternetAddress *ia)
@@ -266,7 +279,10 @@ internet_address_get_type (InternetAddress *ia)
  * internet_address_get_name:
  * @ia: internet address
  *
- * Returns the name of the internet address.
+ * Gets the name component of the internet address. If the internet
+ * address is a group, it will get the group name.
+ *
+ * Returns the name of @ia.
  **/
 const char *
 internet_address_get_name (InternetAddress *ia)
@@ -281,7 +297,9 @@ internet_address_get_name (InternetAddress *ia)
  * internet_address_get_addr:
  * @ia: internet address
  *
- * Returns the address of the internet address.
+ * Gets the addr-spec of the internet address.
+ *
+ * Returns the address of @ia.
  **/
 const char *
 internet_address_get_addr (InternetAddress *ia)
@@ -297,7 +315,10 @@ internet_address_get_addr (InternetAddress *ia)
  * internet_address_get_members:
  * @ia: internet address
  *
- * Returns the list of internet addresses.
+ * Gets the #InternetAddressList containing the group members of an
+ * rfc822 group address.
+ *
+ * Returns the members of @ia.
  **/
 const InternetAddressList *
 internet_address_get_members (InternetAddress *ia)
@@ -314,7 +335,7 @@ internet_address_get_members (InternetAddress *ia)
  * @list: a list of internet addresses
  * @ia: internet address to prepend
  *
- * Prepends the internet address to the list of internet addresses
+ * Prepends the internet address @ia to the list of internet addresses
  * pointed to by @list.
  *
  * Returns the resultant list.
@@ -328,8 +349,8 @@ internet_address_list_prepend (InternetAddressList *list, InternetAddress *ia)
 	
 	internet_address_ref (ia);
 	node = g_new (InternetAddressList, 1);
-	node->next = list;
 	node->address = ia;
+	node->next = list;
 	
 	return node;
 }
@@ -503,18 +524,167 @@ encoded_name (const char *raw, gboolean rfc2047_encode)
 	return name;
 }
 
+enum {
+	INTERNET_ADDRESS_ENCODE = 1 << 0,
+	INTERNET_ADDRESS_FOLD   = 1 << 1,
+};
+
+static void _internet_address_list_to_string (const InternetAddressList *list, guint32 flags, size_t *linelen, GString *string);
+
 static void
-internet_address_list_to_string_internal (const InternetAddressList *list, gboolean encode, GString *string)
+linewrap (GString *string)
+{
+	if (string->len > 0 && string->str[string->len - 1] == ' ') {
+		string->str[string->len - 1] = '\n';
+		g_string_append_c (string, '\t');
+	} else {
+		g_string_append (string, "\n\t");
+	}
+}
+
+static void
+append_folded_name (GString *string, size_t *linelen, const char *name)
+{
+	const char *word, *lwsp;
+	size_t len;
+	
+	word = name;
+	
+	while (*word) {
+		lwsp = word;
+		
+		if (*word == '"') {
+			/* quoted string, don't break these up */
+			lwsp++;
+			
+			while (*lwsp && *lwsp != '"') {
+				if (*lwsp == '\\')
+					lwsp++;
+				
+				if (*lwsp)
+					lwsp++;
+			}
+			
+			if (*lwsp == '"')
+				lwsp++;
+		} else {
+			/* normal word */
+			while (*lwsp && !is_lwsp (*lwsp))
+				lwsp++;
+		}
+		
+		len = lwsp - word;
+		if (*linelen > 1 && (*linelen + len) > GMIME_FOLD_LEN) {
+			linewrap (string);
+			*linelen = 1;
+		}
+		
+		g_string_append_len (string, word, len);
+		*linelen += len;
+		
+		word = lwsp;
+		while (*word && is_lwsp (*word))
+			word++;
+		
+		if (*word && is_lwsp (*lwsp)) {
+			g_string_append_c (string, ' ');
+			(*linelen)++;
+		}
+	}
+}
+
+static void
+_internet_address_to_string (const InternetAddress *ia, guint32 flags, size_t *linelen, GString *string)
+{
+	gboolean encode = flags & INTERNET_ADDRESS_ENCODE;
+	gboolean fold = flags & INTERNET_ADDRESS_FOLD;
+	char *name;
+	size_t len;
+	
+	if (ia->type == INTERNET_ADDRESS_NAME) {
+		if (ia->name && *ia->name) {
+			name = encoded_name (ia->name, encode);
+			len = strlen (name);
+			
+			if (fold && (*linelen + len) > GMIME_FOLD_LEN) {
+				if (len > GMIME_FOLD_LEN) {
+					/* we need to break up the name */
+					append_folded_name (string, linelen, name);
+				} else {
+					/* the name itself is short enough to fit on a single
+					 * line, but only if we write it on a line by itself */
+					if (*linelen > 1) {
+						linewrap (string);
+						*linelen = 1;
+					}
+					
+					g_string_append_len (string, name, len);
+					*linelen += len;
+				}
+			} else {
+				/* we can safly fit the name on this line */
+				g_string_append_len (string, name, len);
+				*linelen += len;
+			}
+			
+			g_free (name);
+			
+			len = strlen (ia->value.addr);
+			
+			if (fold && (*linelen + len + 3) >= GMIME_FOLD_LEN) {
+				g_string_append_len (string, "\n\t<", 3);
+				*linelen = 2;
+			} else {
+				g_string_append_len (string, " <", 2);
+				*linelen += 2;
+			}
+			
+			g_string_append_len (string, ia->value.addr, len);
+			g_string_append_c (string, '>');
+			*linelen += len + 1;
+		} else {
+			len = strlen (ia->value.addr);
+			
+			if (fold && (*linelen + len) > GMIME_FOLD_LEN) {
+				linewrap (string);
+				*linelen = 1;
+			}
+			
+			g_string_append_len (string, ia->value.addr, len);
+			*linelen += len;
+		}
+	} else if (ia->type == INTERNET_ADDRESS_GROUP) {
+		InternetAddressList *members;
+		
+		name = encoded_name (ia->name, encode);
+		len = strlen (name);
+		
+		if (fold && *linelen > 1 && (*linelen + len + 1) > GMIME_FOLD_LEN) {
+			linewrap (string);
+			*linelen = 1;
+		}
+		
+		g_string_append_len (string, name, len);
+		g_string_append_len (string, ": ", 2);
+		*linelen += len + 2;
+		g_free (name);
+		
+		members = ia->value.members;
+		_internet_address_list_to_string (members, flags, linelen, string);
+		g_string_append_c (string, ';');
+		*linelen += 1;
+	}
+}
+
+static void
+_internet_address_list_to_string (const InternetAddressList *list, guint32 flags, size_t *linelen, GString *string)
 {
 	while (list) {
-		char *addr;
+		_internet_address_to_string (list->address, flags, linelen, string);
 		
-		addr = internet_address_to_string (list->address, encode);
-		if (addr) {
-			g_string_append (string, addr);
-			g_free (addr);
-			if (list->next)
-				g_string_append (string, ", ");
+		if (list->next) {
+			g_string_append (string, ", ");
+			*linelen += 2;
 		}
 		
 		list = list->next;
@@ -536,32 +706,16 @@ internet_address_list_to_string_internal (const InternetAddressList *list, gbool
 char *
 internet_address_to_string (const InternetAddress *ia, gboolean encode)
 {
-	char *name, *str = NULL;
+	guint32 flags = encode ? INTERNET_ADDRESS_ENCODE : 0;
+	size_t linelen = 0;
+	GString *string;
+	char *str;
 	
-	if (ia->type == INTERNET_ADDRESS_NAME) {
-		if (ia->name && *ia->name) {
-			name = encoded_name (ia->name, encode);
-			str = g_strdup_printf ("%s <%s>", name, ia->value.addr);
-			g_free (name);
-		} else {
-			str = g_strdup (ia->value.addr);
-		}
-	} else if (ia->type == INTERNET_ADDRESS_GROUP) {
-		InternetAddressList *members;
-		GString *string;
-		
-		name = encoded_name (ia->name, encode);
-		string = g_string_new (name);
-		g_string_append (string, ": ");
-		g_free (name);
-		
-		members = ia->value.members;
-		internet_address_list_to_string_internal (members, encode, string);
-		g_string_append (string, ";");
-		
-		str = string->str;
-		g_string_free (string, FALSE);
-	}
+	string = g_string_new ("");
+	_internet_address_to_string (ia, flags, &linelen, string);
+	str = string->str;
+	
+	g_string_free (string, FALSE);
 	
 	return str;
 }
@@ -580,17 +734,38 @@ internet_address_to_string (const InternetAddress *ia, gboolean encode)
 char *
 internet_address_list_to_string (const InternetAddressList *list, gboolean encode)
 {
+	guint32 flags = encode ? INTERNET_ADDRESS_ENCODE : 0;
+	size_t linelen = 0;
 	GString *string;
 	char *str;
 	
 	string = g_string_new ("");
-	internet_address_list_to_string_internal (list, encode, string);
+	_internet_address_list_to_string (list, flags, &linelen, string);
 	str = string->str;
+	
 	g_string_free (string, FALSE);
 	
 	return str;
 }
 
+
+/**
+ * internet_address_list_writer:
+ * @list: list of internet addresses
+ * @str: string to write to
+ *
+ * Writes the rfc2047-encoded rfc822 formatted addresses in @list to
+ * @string, folding appropriately.
+ **/
+void
+internet_address_list_writer (const InternetAddressList *list, GString *str)
+{
+	guint32 flags = INTERNET_ADDRESS_ENCODE | INTERNET_ADDRESS_FOLD;
+	size_t linelen = str->len;
+	
+	_internet_address_list_to_string (list, flags, &linelen, str);
+}
+
 static InternetAddress *
 decode_mailbox (const char **in)
 {
@@ -618,8 +793,7 @@ decode_mailbox (const char **in)
 			g_string_append (name, pre);
 			g_free (pre);
 		retry:
-			pre = decode_word (&inptr);
-			if (pre)
+			if ((pre = decode_word (&inptr)))
 				g_string_append_c (name, ' ');
 		}
 		
@@ -629,7 +803,7 @@ decode_mailbox (const char **in)
 			bracket = TRUE;
 			pre = decode_word (&inptr);
 		} else if (!retried && *inptr) {
-			w(g_warning ("Unexpected char '%c' in address: %s: attempting recovery.",
+			w(g_warning ("Unexpected char '%c' in mailbox: %s: attempting recovery.",
 				     *inptr, *in));
 			/* chew up this bad char and then attempt 1 more pass at parsing */
 			g_string_append_c (name, *inptr++);
@@ -732,21 +906,12 @@ decode_mailbox (const char **in)
 	
 	if (addr->len) {
 		if (name && !g_utf8_validate (name->str, -1, NULL)) {
-			/* A (broken) mailer has sent us an unencoded
-			 * 8bit value (and it doesn't seem to be valid
-			 * UTF-8 either).  Attempt to save it by
-			 * assuming it's in the user's locale and
-			 * converting to UTF-8 */
-			char *buf;
+			/* A (broken) mailer has sent us raw 8bit/multibyte text data... */
+			char *utf8 = g_mime_utils_decode_8bit (name->str, name->len);
 			
-			if ((buf = g_mime_iconv_locale_to_utf8 (name->str))) {
-				g_string_truncate (name, 0);
-				g_string_append (name, buf);
-				g_free (buf);
-			} else {
-				d(g_warning ("Failed to convert \"%s\" to UTF-8: %s",
-					     name->str, g_strerror (errno)));
-			}
+			g_string_truncate (name, 0);
+			g_string_append (name, utf8);
+			g_free (utf8);
 		}
 		
 		mailbox = internet_address_new_name (name ? name->str : NULL, addr->str);
@@ -777,8 +942,7 @@ decode_address (const char **in)
 		g_string_append (name, pre);
 		g_free (pre);
 		
-		pre = decode_word (&inptr);
-		if (pre)
+		if ((pre = decode_word (&inptr)))
 			g_string_append_c (name, ' ');
 	}
 	
